
######################################################
######################################################
#            REGRESSION KERNEL ESTIMATION            #
######################################################
######################################################

##-- Attaching the data.

data(airquality)
names(airquality)



##-- Positions where Ozones value is NA.

ind_1<-which(is.na(airquality$Ozone))
ind_2<-order(airquality$Temp[-ind_1])

Ozn<-airquality$Ozone[-ind_1]
Tmp<-airquality$Temp[-ind_1]


######################################################
# 1.- Analisys of univariant distribution            #
######################################################

##-- Temperature time series.

windows()
temp.ts<-ts(airquality$Temp)
plot(temp.ts, type="o", ylab="Temperatura",xlab="Da",main="Serie de tiempo de la temperatura")


##-- Ozone and Temperature Empirical Distribution Functions 

windows()
par(mfrow=c(1,2))
plot(ecdf(Ozn),main="Ozone")
plot(ecdf(Tmp),main="Temperature")


##-- Ozone and Temperature density functions 


kern_est<-function(x,h,mas){
    return(1/(length(mas)*h)*t(matrix(apply(dnorm(apply(outer(x,mas,"-"),2,function(x)outer(x,h,"/"))),1,sum),ncol=length(h))))
    

}

LCV_sub<-function(h,mas,tri_matrix_sub){
    return(sum(dnorm(outer(mas,mas,"-")/h,sd=sqrt(2)))/(length(mas)^2*h)
    - (4/(length(mas)*(length(mas)-1)*h)) * sum(dnorm((outer(mas,mas,"-")/h)[tri_matrix_sub])))
}

main<-function(mas,subint,grid,tri_matrix_sub){

opt<- apply(subint,1,function(y) as.numeric(optimize(LCV_sub,interval=c(y[1],y[2]),mas=mas,tri_matrix=tri_matrix_sub)))
h<-opt[1,][which.min(opt[2,])]
est<- kern_est(grid,h,mas)

return(as.numeric(est))

}

#s<-seq(0,8,by=0.5)
s<-cbind(c(0.5,seq(0.5,8,by=0.5)),c(seq(0.5,8,by=0.5),0.5))[-1,][-length(seq(0.5,8,by=0.5)),]

grid<-seq(min(Tmp),max(Tmp),by=0.5)
subint<-cbind(c(0,s),c(s,0))[-1,][-length(s),]
tri_matrix_sub<-upper.tri(matrix(0,nr=length(Tmp),nc=length(Tmp)))
mas<-Tmp
estimador<-main(mas,subint,grid,tri_matrix_sub)


par(mfrow=c(1,2))
hist(Ozn,freq=F,main="Densidad del ozono")
lines(density(Ozn),main="Ozono",col="blue")
hist(Tmp,freq=F,main="Densidad de la temperatura")
lines(density(Tmp),main="Temperatura",col="blue")

par(mfrow=c(1,2))
hist(Ozn,freq=F,main="Densidad del ozono")
lines(density(Ozn),main="Ozono",col="blue")
hist(Tmp,freq=F,main="Densidad de la temperatura")
lines(density(Tmp),main="Temperatura",col="blue")


windows() 
par(mfrow=c(1,2))
hist(Ozn,freq=F,main="Densidad del ozono")
lines(density(Ozn),main="Ozono",col="blue")
hist(Tmp,freq=F,main="Densidad de la temperatura")
lines(density(Tmp),main="Temperatura",col="blue")


##-- Shapiro-Wilks tests 
shapiro.test(Ozn)
shapiro.test(Tmp)




##-- Regression model.

windows()
lm(Ozn~Tmp)
plot(lm(Ozn~Tmp))
windows()
plot(Tmp,Ozn,main="Regression model")
abline(lm(Ozn~Tmp),lwd=2,col="blue")
 

##-- Ozone and Temperature box-plots
windows()
par(mfrow=c(1,2))
boxplot(airquality$Ozone[-ind1],main="Ozone")
boxplot(airquality$Temp[-ind1],main="Temperature")


##-- Ozone and Temperature descriptions
summary(airquality$Ozone[-ind1])
summary(airquality$Temp[-ind1])

#######################################################
# 2.- Local-linear estimator                          #
#######################################################


evals<-function(x,t,h){

# Description
#

# Arguments
#

# Output
#

    ev_1<-outer(x,t,"-")
    ev_2<-apply(ev_1,2,function(x)outer(x,h,"/"))
    return(list(ev_1,ev_2))
}


S<-function(x,r,mat,mat2){

# Description
#

# Arguments
#

# Output
#

#-- x values of the independ variable.
#-- bandwidth window 
#-- r exponent.

    return(apply(matrix(mat*mat2^r,nr=length(x)),2,sum))

}


l<-function(x,t,h,m,opt_m,opt_cv,ev){

# Description
#

# Arguments
#

# Output
#

    mat_1<-matrix(t(dnorm(ev[[2]])),nc=length(t),byrow=T)
    b<- t(matrix(mat_1,nr=length(x)))

    if(opt_m==1){

        mat_2<- matrix(t(ev[[1]]),nc=length(t),nr=length(x)*length(h),byrow=T)
        b<-b*(S(x,2,mat_1,mat_2) - t(matrix(mat_2,nr=length(x)))* S(x,1,mat_1,mat_2))
        l_ll<-b/apply(b,1,sum)

        if(opt_cv==1){

            b_diag<-matrix(t(l_ll),nr=length(x)*length(h))
            l_diag<-matrix(b_diag[m==T],nc=length(h),byrow=T)

            return(list(l_ll,l_diag))
        }

        else{
            return(l_ll)
        }

    }
    else{
    
        l_nw<-b/apply(b,1,sum)

        if(opt_cv==1){

            b_diag<-matrix(t(l_nw),nr=length(x)*length(h))
            l_diag<-matrix(b_diag[m==T],nc=length(h),byrow=T)

            return(list(l_nw,l_diag))
        }
        else{
        return(l_nw)
        }
    
    }
        
}

main<-function(t,x,y,h){

# Description
#

# Arguments
#

# Output
#

    ## Local - linear

    ev<-evals(x,t,h)

    ll_cv<-l(t,x,h,mat,1,1,ev)

    m_ll<-matrix(ll_cv[[1]]%*%y,nc=length(h),byrow=T)

    cv_ll<-1/length(x) * apply(( (matrix(y,nr=length(x),nc=length(h)) - m_ll ) / (1 - ll_cv[[2]]) )^2 ,2,sum)
    h_cv_ll<-h[which.min(cv_ll)]
    
    windows()
    plot(cv_ll~h,type="l",lwd=2,main="CV para E.L.L.",ylab="CV(h)")

    windows()
    plot(Tmp[ind_2],Ozn[ind_2],xlab="Temperatura", ylab="Ozono",main="Estimacin por el E.L.L")
    lines(m_ll[,which.min(cv_ll)][ind_2]~Tmp[ind_2],lwd=2)

    ## Nadaraya - Watson 

    nw<-l(mas,mas,h,mat,2,1,ev)

    m_nw<-matrix(nw[[1]]%*%y,nc=length(h),byrow=T)

    cv_nw<-1/length(mas) * apply( ( ( matrix(y,nr=length(mas),nc=length(h)) - m_nw ) / (1 - nw[[2]]) )^2 ,2,sum)
    h_cv_nw<-h[which.min(cv)]

    windows()
    plot(cv_nw~h,type="l",main="Cross Validaton Nadaraya-Watson")

    windows()
    plot(Tmp[ind_2],Ozn[ind_2],xlab="Temperatura", ylab="Ozono",main="Comparacin L.L - N.W")
    lines(m_ll[,which.min(cv_ll)][ind_2]~Tmp[ind_2],lwd=2)
    lines(m_nw[,which.min(cv_nw)][ind_2]~Tmp[ind_2],lwd=2,col="red")
    legend(60,150,legend=c("L.L.","N.W"),lwd=c(2,2),col=c("black","red"))

    ## Plug - in
    
    library(KernSmooth)

    h_pi<- dpill(airquality$Temp[-ind],airquality$Ozone[-ind])

    ll_dpill<-l(mas,mas,h_pi,mat,1,0,ev)

    m_pi<-ll_dpill%*%y
    
    windows()
    plot(Tmp[ind_2],Ozn[ind_2],xlab="Temperatura", ylab="Ozone",main="Comparacin L.L. con ventanas C.V. y P.I.")
    lines(m_ll[,which.min(cv_ll)][ind_2]~Tmp[ind_2],lwd=2)
    lines(m_pi[ind_2]~Tmp[ind_2],lwd=2,col="blue")
    legend(60,150,legend=c("C.V.","P.I."),lwd=c(2,2),col=c("black","blue"))


}


## Results

h<-seq(0.5,5,by=0.02)
mat<-matrix(diag(T,nr=length(mas),nc=length(mas)),nc=length(mas),nr=length(mas)*length(h),byrow=T)
results<-main(Tmp,Tmp,Ozn,h,mat)
    




##-- Without outliers: 
# Observations 57,70,83. Sample in order
plot(lm(airquality$Temp[-ind][order(airquality$Temp[-ind])]~airquality$Ozone[-ind][order(airquality$Temp[-ind])]))

windows()
par(mfrow=c(1,2))


## Local-linear estimator using cross-validation ##

##-- Sequency of windows
h<-seq(0,20,by=0.02)

##--Save the values of the cross-validation function aplicated to no-parametric estimator
#-- of local-linear regresion.
CV_ll<-numeric()
for(i in 1:length(h)){CV_ll[i]<-ll_cv(airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Ozone[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],h[i])}

##-- Graphic representacion of of the ross-validation function aplicated to no-parametric estimator
#-- of local-linear regresion respect to the sequuency of windows.
plot(CV_ll~h,type="l",main="cross validaton Linear-local")

##-- Estimator of local-linear regresion with the cross-validation bandwidth.
local_linear_without<-EstLL(airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Ozone[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],h[which(CV_ll==min(CV_ll,na.rm=T))])


## Local-linear estimator using plug-in ##

##--Local linear Gaussian kernel regression estimate.
##--Use direct plug-in methodology to select the bandwidth of a local linear Gaussian kernel regression estimate.
local_linear_plugin_without<-EstLL(airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Ozone[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],dpill(airquality$Temp[-ind],airquality$Ozone[-ind]))


## Nadaraya-Watson estimation using cross-validation ##

##-- Windows sequency
h<-seq(0,20,by=0.2)

##--Save the values of the ross-validation function aplicated to no-parametric estimator
#-- of Nadaraya-Watson regresion.
CV_nw<-numeric()
for(i in 1:length(h)){CV_nw[i]<-cv_nw(airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Ozone[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],h[i])}

##-- Graphic representacion of of the ross-validation function aplicated to no-parametric estimator
#- of Nadaraya-Watson regresion respect to the sequency of windows.
plot(CV_nw~h,type="l",main="cross validation Nadaraya-Watson")

##-- Estimator of Nadaraya-Watson regresion with the cross-validation bandwidth.
nadaraya_watson_without<-nw(airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Ozone[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],h[which(CV_nw==min(CV_nw,na.rm=T))])



windows()
par(mfrow=c(2,2))

plot(airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Ozone[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],xlab="Temperatura without outliers", ylab="Ozone without outliers",main="Local linear CV")
lines(local_linear_without~airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],lwd=2)

plot(airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Ozone[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],xlab="Temperature without outliers", ylab="Ozone without outliers",main="Local linear plug-in")
lines(local_linear_plugin_without~airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],col="red",lwd=2)

plot(airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],airquality$Ozone[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],xlab="Temperature without outliers", ylab="Ozone without outliers",main="Nadaraya-Watson estimator")
lines(nadaraya_watson_without~airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],col="blue",lwd=2)


##########################################################
#   5.- Sample model: Piecewise Linear Regression        #
##########################################################

windows()
par(mfrow=c(2,2))
plot(airquality$Temp[-ind][order(airquality$Temp[-ind])],airquality$Ozone[-ind][order(airquality$Temp[-ind])],xlab="Temperature",ylab="Ozone", main="Local-lineal estimator")
legend(x=60, y=100, legend = c("Plug-in","Cross-validation"), col=c("blue", "green"), pch=1, bty="n")

lines(airquality$Temp[-ind][order(airquality$Temp[-ind])][1:max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==75))],lm(airquality$Ozone[-ind][order(airquality$Temp[-ind])][1:max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==75))]~airquality$Temp[-ind][order(airquality$Temp[-ind])][1:max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==75))])$fitted.values,lwd=2,col="blue")

lines(airquality$Temp[-ind][order(airquality$Temp[-ind])][max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==75)):max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==91))],lm(airquality$Ozone[-ind][order(airquality$Temp[-ind])][max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==75)):max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==91))]~airquality$Temp[-ind][order(airquality$Temp[-ind])][max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==75)):max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==91))])$fitted.values,lwd=2,col="blue")

lines(airquality$Temp[-ind][order(airquality$Temp[-ind])][max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==91)):max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==96))],lm(airquality$Ozone[-ind][order(airquality$Temp[-ind])][max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==91)):max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==96))]~airquality$Temp[-ind][order(airquality$Temp[-ind])][max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==91)):max(which(airquality$Temp[-ind][order(airquality$Temp[-ind])]==96))])$fitted.values,lwd=2,col="blue")

lines(local_linear_without~airquality$Temp[-ind][order(airquality$Temp[-ind])][-c(57,70,83)],lwd=2,col="green")


##-Whith splines


library(splines)
##-- This package provides functions for working with regression splines using the B-spline basis, bs, and the natural cubic spline basis, ns. 

##-- function bs: Generate the B-spline basis matrix for a polynomial spline
##-- ARGUMENTS
#- degree: degree of the piecewise polynomial?default is 3 for cubic splines.
#- knots:  the internal breakpoints that define the spline. The default is NULL, which results in a basis for ordinary polynomial regression. 
#-         Typical values are the mean or median for one knot, quantiles for more knots. 


lm(Ozone~bs(Temp,knots=c(75,91,96),degree=1),data=airquality)
summary(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=1),data=airquality))
plot(airquality$Temp,airquality$Ozone, xlab="Temperature", ylab="Ozone",
main="Piecewise Linear Regression 1")
new <- data.frame(Temp = seq(min(airquality$Temp),max(airquality$Temp),1))
lines(new$Temp[-length(new$Temp)],predict(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=1,df=3),data=airquality),new)[-length(predict(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=1,df=3),data=airquality),new))],lty=1,col="red",lwd=3)


lm(Ozone~bs(Temp,knots=c(75,91,96),degree=2),data=airquality)
summary(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=2),data=airquality))
plot(airquality$Temp,airquality$Ozone, xlab="Temperature", ylab="Ozone",
main="Piecewise Linear Regression 2")
new <- data.frame(Temp = seq(min(airquality$Temp),max(airquality$Temp),1))
lines(new$Temp[-length(new$Temp)],predict(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=2),data=airquality),new)[-length(predict(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=1,df=3),data=airquality),new))],lty=1,col="red",lwd=3)


lm(Ozone~bs(Temp,knots=c(75,91,96),degree=3),data=airquality)
summary(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=3),data=airquality))
plot(airquality$Temp,airquality$Ozone, xlab="Temperature", ylab="Ozone",
main="Piecewise Linear Regression 3")
new <- data.frame(Temp = seq(min(airquality$Temp),max(airquality$Temp),1))
lines(new$Temp[-length(new$Temp)],predict(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=3),data=airquality),new)[-length(predict(lm(Ozone~bs(Temp,knots=c(75,91,96),degree=1,df=3),data=airquality),new))],lty=1,col="red",lwd=3)
